@@ -2,7 +2,7 @@ package Astro::FITS::Header;
 
 # ---------------------------------------------------------------------------
 
-#+
+#+ 
 #  Name:
 #    Astro::FITS::Header
 
@@ -21,10 +21,10 @@ package Astro::FITS::Header;
 #    Alasdair Allan (aa@astro.ex.ac.uk)
 #    Tim Jenness (t.jenness@jach.hawaii.edu)
 #    Craig DeForest (deforest@boulder.swri.edu)
-#    Jim Lewis (jrl@ast.cam.ac.uk)
 
 #  Revision:
-#     $Id: Header.pm,v 1.26 2003/10/23 03:28:28 bradc Exp $
+#     $Id: Header.pm,v 1.12 2002/06/10 19:30:06 allan Exp $
+     
 
 #  Copyright:
 #     Copyright (C) 2001-2002 Particle Physics and Astronomy Research Council. 
@@ -37,7 +37,7 @@ package Astro::FITS::Header;
 
 =head1 NAME
 
-Astro::FITS::Header - Object Orientated interface to FITS HDUs
+Astro::FITS::Header - A FITS header
 
 =head1 SYNOPSIS
 
@@ -58,17 +58,13 @@ use vars qw/ $VERSION /;
 
 use Astro::FITS::Header::Item;
 
-'$Revision: 1.26 $ ' =~ /.*:\s(.*)\s\$/ && ($VERSION = $1);
-
-# Operator overloads
-use overload '""' => "stringify",
-  fallback => 1;
+'$Revision: 1.12 $ ' =~ /.*:\s(.*)\s\$/ && ($VERSION = $1);
 
 # C O N S T R U C T O R ----------------------------------------------------
 
 =head1 REVISION
 
-$Id: Header.pm,v 1.26 2003/10/23 03:28:28 bradc Exp $
+$Id: Header.pm,v 1.12 2002/06/10 19:30:06 allan Exp $
 
 =head1 METHODS
 
@@ -82,9 +78,7 @@ Create a new instance from an array of FITS header cards.
 
   $item = new Astro::FITS::Header( Cards => \@header );
 
-returns a reference to a Header object.  If you pass in no cards, 
-you get the (required) first SIMPLE card for free.
-
+returns a reference to a Header object.
 
 =cut
 
@@ -95,14 +89,10 @@ sub new {
   # bless the header block into the class
   my $block = bless { HEADER => [],
                       LOOKUP  => {},
-		      LASTKEY => undef,
-		      TieRetRef => 0,
-                      SUBHDRS => [],
-		    }, $class;
+		      LASTKEY => undef }, $class;
 
-  # Configure the object, even with no arguments since configure
-  # still puts the minimum SIMPLE card in.
-  $block->configure( @_ );
+  # If we have arguments configure the object
+  $block->configure( @_ ) if @_;
 
   return $block;
 
@@ -116,52 +106,6 @@ sub new {
 
 =over 4
 
-=item B<tieRetRef>
-
-Indicates whether the tied object should return multiple values
-as a single string joined by newline characters (false) or 
-it should return a reference to an array containing all the values.
-
-Only affects the tied interface.
-
-  tie %keywords, "Astro::FITS::Header", $header, tiereturnsref => 1;
-  $ref = $keywords{COMMENT};
-
-Defaults to returning a single string in all cases (for backwards
-compatibility)
-
-=cut
-
-sub tiereturnsref {
-  my $self = shift;
-  if (@_) {
-    $self->{TieRetRef} = shift;
-  }
-  return $self->{TieRetRef};
-}
-
-=item B<subhdrs>
-
-Set or return the subheaders for a Header object
-
-    $header->subhdrs(@hdrs);
-    @hdrs = $header->subhdrs;
-
-=cut
-
-sub subhdrs {
-    my $self = shift;
-
-    if (@_) {
-        @{$self->{SUBHDRS}} = @_;
-    }
-    if (wantarray()) {
-        return @{$self->{SUBHDRS}};
-    } else {
-        return $self->{SUBHDRS};
-    }
-}
-
 =item B<item>
 
 Returns a FITS::Header:Item object referenced by index, C<undef> if it
@@ -173,12 +117,13 @@ does not exist.
 
 sub item {
    my ( $self, $index ) = @_;
-
+   
    return undef unless defined $index;
    return undef unless exists ${$self->{HEADER}}[$index];
-
+   
    # grab and return the Header::Item at $index
    return ${$self->{HEADER}}[$index];
+ 
 }
 
 # K E Y W O R D ------------------------------------------------------------
@@ -193,12 +138,13 @@ Returns keyword referenced by index, C<undef> if it does not exist.
 
 sub keyword {
    my ( $self, $index ) = @_;
-
+   
    return undef unless defined $index;
    return undef unless exists ${$self->{HEADER}}[$index];
-
+   
    # grab and return the keyword at $index
    return ${$self->{HEADER}}[$index]->keyword();
+ 
 }
 
 # I T E M   B Y   N A M E  -------------------------------------------------
@@ -207,8 +153,7 @@ sub keyword {
 
 Returns an array of Header::Items for the requested keyword if called
 in list context, or the first matching Header::Item if called in scalar
-context. Returns C<undef> if the keyword does not exist.  The keyword
-may be a regular expression created with the C<qr> operator.
+context. Returns C<undef> if the keyword does not exist.
 
    @items = $header->itembyname($keyword);
    $item = $header->itembyname($keyword);
@@ -219,20 +164,25 @@ may be a regular expression created with the C<qr> operator.
 
 sub itembyname {
    my ( $self, $keyword ) = @_;
-
-   my @items = @{$self->{HEADER}}[$self->index($keyword)];
-
+            
+   # resolve the items from the index array from lookup table
+   # grab the index array from the lookup table
+   my @index;
+   @index = @{${$self->{LOOKUP}}{$keyword}}
+         if ( exists ${$self->{LOOKUP}}{$keyword} && 
+	      defined ${$self->{LOOKUP}}{$keyword} );
+   my @items = map {${$self->{HEADER}}[$_]} @index;
+   
    return wantarray ?  @items : @items ? $items[0] : undef;
-
+   
 }
 
 # I N D E X   --------------------------------------------------------------
 
 =item B<index>
 
-Returns an array of indices for the requested keyword if called in
-list context, or an empty array if it does not exist.  The keyword may
-be a regular expression created with the C<qr> operator.
+Returns an array of indices for the requested keyword if called in list
+context, or an empty array if it does not exist.
 
    @index = $header->index($keyword);
 
@@ -248,20 +198,9 @@ sub index {
    
    # grab the index array from lookup table
    my @index;
-
-   if ( 'Regexp' eq ref $keyword )
-   {
-     push @index, @{$self->{LOOKUP}{$_}}
-       foreach grep { /$keyword/ && 
-			defined $self->{LOOKUP}{$_} } keys %{$self->{LOOKUP}};
-     @index = sort @index;
-   }
-   else
-   {
-     @index = @{${$self->{LOOKUP}}{$keyword}}
-       if ( exists ${$self->{LOOKUP}}{$keyword} && 
-	    defined ${$self->{LOOKUP}}{$keyword} );
-   }
+   @index = @{${$self->{LOOKUP}}{$keyword}}
+         if ( exists ${$self->{LOOKUP}}{$keyword} && 
+	      defined ${$self->{LOOKUP}}{$keyword} );
    
    # return the values array
    return wantarray ? @index : @index ? $index[0] : undef;
@@ -272,9 +211,8 @@ sub index {
 
 =item B<value>
 
-Returns an array of values for the requested keyword if called in list
-context, or an empty array if it does not exist.  The keyword may be
-a regular expression created with the C<qr> operator.
+Returns an array of values for the requested keyword if called
+in list context, or an empty array if it does not exist.
 
    @value = $header->value($keyword);
 
@@ -285,13 +223,16 @@ C<undef> if the keyword does not exist.
 
 sub value {
    my ( $self, $keyword ) = @_;
-
+   
    # resolve the values from the index array from lookup table
-   my @values = map { ${$self->{HEADER}}[$_]->value() } $self->index($keyword);
+   my @values =
+     map { ${$self->{HEADER}}[$_]->value() } @{${$self->{LOOKUP}}{$keyword}}
+         if ( exists ${$self->{LOOKUP}}{$keyword} && 
+	      defined ${$self->{LOOKUP}}{$keyword} );
 
    # loop over the indices and grab the values
    return wantarray ? @values : @values ? $values[0] : undef;
-
+   
 }
 
 # C O M M E N T -------------------------------------------------------------
@@ -299,8 +240,7 @@ sub value {
 =item B<comment>
 
 Returns an array of comments for the requested keyword if called
-in list context, or an empty array if it does not exist.  The keyword
-may be a regular expression created with the C<qr> operator.
+in list context, or an empty array if it does not exist.
 
    @comment = $header->comment($keyword);
 
@@ -316,7 +256,9 @@ sub comment {
       
    # resolve the comments from the index array from lookup table
    my @comments =
-     map { ${$self->{HEADER}}[$_]->comment() } $self->index($keyword);
+     map { ${$self->{HEADER}}[$_]->comment() } @{${$self->{LOOKUP}}{$keyword}}
+         if ( exists ${$self->{LOOKUP}}{$keyword} && 
+	      defined ${$self->{LOOKUP}}{$keyword} );
    
    # loop over the indices and grab the comments
    return wantarray ?  @comments : @comments ? $comments[0] : undef;
@@ -338,21 +280,13 @@ modify all inserted copies.
 
 sub insert{
    my ($self, $index, $item) = @_;
-
-   # If the array is empty and we get a negative index we
-   # must convert it to an index of 0 to prevent a:
-   #   Modification of non-creatable array value attempted, subscript -1
-   # fatal error
-   # This can occur with a tied hash and the %{$tieref} = %new
-   # construct
-   $index = 0 if (scalar(@{$self->{HEADER}} == 0 && $index < 0));
-
+   
    # splice the new FITS header card into the array
    splice @{$self->{HEADER}}, $index, 0, $item;
-
+   
    # rebuild the lookup table from the modified header
-   $self->_rebuild_lookup();
-
+   $self->_rebuild_lookup();   
+   
 }
 
 
@@ -416,8 +350,7 @@ Replace FITS header cards with keyword $keyword with card $item
 
    $card = $header->replacebyname($keyword, $item);  
 
-returns the replaced card. The keyword may be a regular expression
-created with the C<qr> operator.
+returns the replaced card.
 
 =cut
 
@@ -425,7 +358,10 @@ sub replacebyname{
    my ($self, $keyword, $item) = @_;
    
    # grab the index array from lookup table
-   my @index = $self->index($keyword);
+   my @index;
+   @index = @{${$self->{LOOKUP}}{$keyword}}
+         if ( exists ${$self->{LOOKUP}}{$keyword} && 
+	      defined ${$self->{LOOKUP}}{$keyword} );
 
    # loop over the keywords
    my @cards = map { splice @{$self->{HEADER}}, $_, 1, $item;} @index;
@@ -446,8 +382,7 @@ Removes a FITS header card object by name
 
   @card = $header->removebyname($keyword);
 
-returns the removed cards.  The keyword may be a regular expression
-created with the C<qr> operator.
+returns the removed cards.
 
 =cut
 
@@ -455,7 +390,10 @@ sub removebyname{
    my ($self, $keyword) = @_;
    
    # grab the index array from lookup table
-   my @index = $self->index($keyword);
+   my @index;
+   @index = @{${$self->{LOOKUP}}{$keyword}}
+         if ( exists ${$self->{LOOKUP}}{$keyword} && 
+	      defined ${$self->{LOOKUP}}{$keyword} );
 
    # loop over the keywords
    my @cards = map { splice @{$self->{HEADER}}, $_, 1; } @index;
@@ -530,9 +468,9 @@ sub cards {
   my $self = shift;
   return map { "$_" } @{$self->{HEADER}};
 }
-
-# A L L I T E M S ---------------------------------------------------------
-
+   
+# A L L I T E M S ---------------------------------------------------------   
+ 
 =item B<allitems>
 
 Returns the header as an array of FITS::Header:Item objects.
@@ -544,8 +482,8 @@ Returns the header as an array of FITS::Header:Item objects.
 sub allitems {
    my $self = shift;
    return map { $_ } @{$self->{HEADER}};
-}
-
+} 
+   
 # C O N F I G U R E -------------------------------------------------------
 
 =back
@@ -556,49 +494,39 @@ sub allitems {
 
 =item B<configure>
 
-Configures the object, takes an array of FITS header cards 
-or an array of Astro::FITS::Header::Item objects as input.
-If you feed in nothing at all, it uses a default array containing
-just the SIMPLE card required at the top of all FITS files.
+Configures the object, takes an array of FITS header cards as input.
 
   $header->configure( Cards => \@array );
-  $header->configure( Items => \@array );
 
 Does nothing if the array is not supplied.
 
 =cut
 
 sub configure {
-    my $self = shift;
-    
-    # grab the argument list
-    my %args = @_;
-    
-    if (exists $args{Cards} && defined $args{Cards}) {
-	
-	# First translate each incoming card into a Item object
-	# Any existing cards are removed
-	@{$self->{HEADER}} = map {
-	    new Astro::FITS::Header::Item( Card => $_ );
-	} @{ $args{Cards} };
-	
+  my $self = shift;
+
+  # return unless we have arguments
+  return undef unless @_;
+
+  # grab the argument list
+  my %args = @_;
+
+  if (defined $args{Cards}) {
+
+    # First translate each incoming card into a Item object
+    # Any existing cards are removed
+    @{$self->{HEADER}} = map {
+      new Astro::FITS::Header::Item( Card => $_ );
+    } @{ $args{Cards} };
+
     # Now build the lookup table. There would be a slight efficiency
-	# gain to include this in a loop over the cards but prefer
-	# to reuse the method for this rather than repeating code
-	$self->_rebuild_lookup;
-	
-    } elsif (exists $args{Items} && defined $args{Items}){
-	# We have an array of Astro::FITS::Header::Items
-	@{$self->{HEADER}} = @{ $args{Items} };
-	$self->_rebuild_lookup;
-    } elsif( !defined($self->{HEADER}) ||  !@{$self->{HEADER}} ) {
-	@{$self->{HEADER}} = (
-	      new Astro::FITS::Header::Item( Card=> "SIMPLE  =  T"),
-	      new Astro::FITS::Header::Item( Card=> "END", Type=>"END" )
-			      );
-	$self->_rebuild_lookup; 
-    }
+    # gain to include this in a loop over the cards but prefer
+    # to reuse the method for this rather than repeating code
+    $self->_rebuild_lookup;
+
+  }
 }
+
 =item B<freeze>
 
 Method to return a blessed reference to the object so that we can store
@@ -615,26 +543,6 @@ sub freeze {
 
 =back
 
-=head2 Operator Overloading
-
-These operators are overloaded:
-
-=over 4
-
-=item B<"">
-
-When the object is used in a string context the FITS header
-block is returned as a single string.
-
-=cut
-
-sub stringify {
-  my $self = shift;
-  return join("\n", $self->cards )."\n";
-}
-
-=back
-
 =head2 Private methods
 
 These methods are for internal use only.
@@ -704,43 +612,18 @@ Header value type is determined on-the-fly by parsing of the input values.
 Anything that parses as a number or a logical is converted to that before
 being put in a card (but see below).
 
-Per-card comment fields can be accessed using the tied interface by specifying
-a key name of "key_COMMENT". This works because in general "_COMMENT" is too
-long to be confused with a normal key name.
-
-  $comment = $hdr{CRPIX1_COMMENT};
-
-will return the comment associated with CRPIX1 header item. The comment
-can be modified in the same way:
-
-  $hdr{CRPIX1_COMMENT} = "An axis";
-
-You can also modify the comment by slash-delimiting it when setting the 
-associated keyword:
-
-  $hdr{CRPIX1} = "34 / Set this field manually";
-
-If you want an actual slash character in your string field you must escape
-it with a backslash.  (If you're in double quotes you have to use a double
-backslash):
-
-  $hdr{SLASHSTR} = 'foo\/bar / field contains "foo/bar"';
+There's no way to access the per-card comment fields using the tied interface.
 
 Keywords are CaSE-inNSEnSiTIvE, unlike normal hash keywords.  All
 keywords are translated to upper case internally, per the FITS standard.
 
-Aside from the SIMPLE and END keywords, which are automagically placed at
-the beginning and end of the header respectively, keywords are included
-in the header in the order received.  This gives you a modicum of control
-over card order, but if you actually care what order they're in, you
-probably don't want the tied interface.
-
 =head2 Comment cards
 
 Comment cards are a special case because they have no normal value and
 their comment field is treated as the hash value.  The keywords
-"COMMENT" and "HISTORY" are magic and refer to comment cards; nearly all other
-keywords create normal valued cards.  (see "SIMPLE and END cards", below).
+"COMMENT" and "HISTORY" are magic and refer to comment cards; all other
+keywords create normal valued cards.  If you don't like that behavior,
+don't use the tied interface.
 
 =head2 Multi-card values
 
@@ -758,7 +641,7 @@ appended automatically on FETCH, so that
   $hash{HISTORY} .= "Added multi-line string support";
 
 adds a new HISTORY comment card, while
-
+  
   $hash{TELESCOP} .= " dome B";
 
 only modifies an existing TELESCOP card.
@@ -786,65 +669,25 @@ back on the non-tied interface by calling methods like so:
 
   ((tied $hash)->method())
 
-If you prefer to have multi-valued items automagically become array
-refs, then you can get that behavior using the C<tiereturnsref> method:
-
-  tie %keywords, "Astro::FITS::Header", $header, tiereturnsref => 1;
-
-When tiereturnsref is true, multi-valued items will be returned via a
-reference to an array (ties do not respect calling context). Note that
-if this is configured you will have to test each return value to see
-whether it is returning a real value or a reference to an array if you
-are not sure whether there will be more than one card with a duplicate
-name.
-
 =head2 Type forcing
 
 Because perl uses behind-the-scenes typing, there is an ambiguity
 between strings and numeric and/or logical values: sometimes you want
 to create a STRING card whose value could parse as a number or as a
 logical value, and perl kindly parses it into a number for you.  To
-force string evaluation, feed in a trivial array ref:
+force string evaluation, feed in a trivial hash array:
 
   $hash{NUMSTR} = 123;     # generates an INT card containing 123.
-  $hash{NUMSTR} = "123";   # generates an INT card containing 123.
-  $hash{NUMSTR} = ["123"]; # generates a STRING card containing "123".
-  $hash{NUMSTR} = [123];   # generates a STRING card containing "123".
+  $hash{NUMSTR} = '123';   # generates an INT card containing 123.
+  $hash{NUMSTR} = ['123']; # generates a STRING card containing '123'.
+  $hash{NUMSTR} = [123];   # generates a STRING card containing '123'.
 
-  $hash{ALPHA} = "T";      # generates a LOGICAL card containing T. 
-  $hash{ALPHA} = ["T"];    # generates a STRING card containing "T".
+  $hash{ALPHA} = 'T';      # generates a LOGICAL card containing T. 
+  $hash{ALPHA} = ['T'];    # generates a STRING card containing 'T'.
 
 Calls to keys() or each() will, by default, return the keywords in the order 
 n which they appear in the header.
 
-When the key refers to a subheader entry, a hash reference is returned.
-If a hash reference is stored in a value it is converted to a
-C<Astro::FITS::Header> object.
-
-=head2 SIMPLE and END cards
-
-No FITS interface would becomplete without special cases.  
- 
-When you assign to SIMPLE or END, the tied interface ensures that they
-are first or last, respectively, in the deck -- as the FITS standard
-requires.  Other cards are inserted in between the first and last
-elements, in the order that you define them.  
-
-The SIMPLE card is forced to FITS LOGICAL (boolean) type.  The FITS
-standard forbids you from setting it to F, but you can if you want --
-we're not the FITS police.
-
-The END card is forced to a null type, so any value you assign to it
-will fall on the floor.  If present in the deck, the END keyword
-always contains the value " ", which is both more-or-less invisible
-when printed and also true -- so you can test the return value to see
-if an END card is present.
-
-SIMPLE and END come pre-defined from the constructor.  If for some
-nefarious reason you want to remove them you must explicitly do so
-with "delete" or the appropriate method call from the object
-interface.
-
 =cut
 
 # List of known comment-type fields
@@ -857,113 +700,31 @@ interface.
 # constructor
 sub TIEHASH {
   my ( $class, $obj, %options ) = @_;
-  my $newobj = bless $obj, $class;
-
-  # Process options
-  for my $key (keys %options) {
-    my $method = lc($key);
-    if ($newobj->can($method)) {
-      $newobj->$method( $options{$key});
-    }
-  }
-
-  return $newobj;
+  return bless $obj, $class;  
 }
 
 # fetch key and value pair
-# MUST return undef if the key is missing else autovivification of 
-# sub header will fail
 
 sub FETCH {
   my ($self, $key) = @_;
-
-  $key = uc($key);
-
-  # If the key has a _COMMENT suffix we are looking for a comment
-  my $wantvalue = 1;
-  if ($key =~ /_COMMENT$/) {
-    $wantvalue = 0;
-    # Remove suffix
-    $key =~ s/_COMMENT$//;
-  }
-
-  # if we are of type COMMENT we want to retrieve the comment regardless
-  # We find this by getting the first item that matches
-  my $item = ($self->itembyname($key))[0];
-  my $t_ok = (defined $item) && (defined $item->type);
-  $wantvalue = 0 if ($t_ok && ($item->type eq 'COMMENT'));
-
-  # The END card is a special case.  We always return " " for the value,
-  # and undef for the comment.
-  return ($wantvalue ? " " : undef)
-      if( ($t_ok && ($item->type eq 'END')) || 
-	  ((defined $item) && ($key eq 'END')) );
-
-  # Retrieve all the values/comments. Note that we go through the entire
-  # header for this in case of multiple matches
-  my @values = ($wantvalue ? $self->value( $key ) : $self->comment($key) );
-
-  # Return value depends on return context. If we have one value it does not
-  # matter, just return it. In list context want all the values, in scalar
-  # context join them all with a \n
-  # Note that in a TIED hash we do not have access to the calling context
-  # we are ALWAYS in scalar context.
-  my @out;
-
-  # Sometimes we want the array to remain an array
-  if ($self->tiereturnsref) {
-    @out = @values;
-  } else {
-
-    # Join everything together with a newline
-    # BUT we are careful here to prevent stringification of references
-    # at least for the case where we only have one value. We also must
-    # handle the case where we have no value to return (without turning
-    # it into a null string since that ruins autovivification of sub headers)
-    if (scalar(@values) <= 1) {
-      @out = @values;
-    } else {
-
-      # Multi values so join [protecting warnings from undef]
-      @out = ( join("\n", map { defined $_ ? $_ : '' } @values) );
-
-      # This is a hangover from the STORE (where we add a \ continuation 
-      # character to multiline strings)
-      $out[0] =~ s/\\\n//gs if (defined($out[0]));
-    }
-  }
-
-  # COMMENT cards get a newline appended.
-  # (Whether this should happen is controversial, but it supports
-  # the "just append a string to get a new COMMENT card" behavior
-  # described in the documentation).
-  if ($t_ok && ($item->type eq 'COMMENT')) {
-    @out = map { $_ . "\n" } @out;
-  }
-
-  # If we have a header we need to tie it to another hash
-  my $ishdr = ($t_ok && $item->type eq 'HEADER');
-  for my $hdr (@out) {
-    if ((UNIVERSAL::isa($hdr, "Astro::FITS::Header")) || $ishdr) {
-      my %header;
-      tie %header, ref($hdr), $hdr;
-      # Change in place
-      $hdr = \%header;
-    }
-  }
-
-  # Can only return a scalar
-  # So return the first value if tiereturnsref is false.
-  # (by this point, all the values should be joined together into the
-  # first element anyway.)
-  my $out;
-  if ($self->tiereturnsref && scalar(@out) > 1) {
-      $out = \@out;
-  } else {
-      $out = $out[0];
-  }
   
+  $key = uc($key);
+  my $item = ($self->itembyname($key))[0];
+  
+  my @values = ((defined $item) && (defined $item->type) && ($item->type eq 'COMMENT')) ?
+      $self->comment($key) :
+	  $self->value($key);
+  
+  my $out;
+  if($#values <= 0) {
+      $out = $values[0];
+  } else {
+      $out = join("\n",@values);
+      $out =~ s/\\\n//gs if (defined($out));
+  }
+  $out .= "\n" if((defined $item) && (defined $item->type) && ($item->type =~ m/^(COMMENT)$/s));
   return $out;
+  
 }
 
 # store key and value pair
@@ -981,64 +742,13 @@ sub FETCH {
 
 sub STORE {
   my ($self, $keyword, $value) = @_;
-  my @values;
-
-  # Recognize slash-delimited comments in value keywords.  This is done 
-  # cheesily via recursion -- would be more efficient, but less readable, 
-  # to propagate the comment through the code...
-
-  # I think this is fundamentally flawed. If I store a string "foo/bar"
-  # in a hash and then read it back I expect to get "foo/bar" not "foo".
-  # I can not be expected to know that this hash happens to be tied to
-  # a FITS header that is trying to spot FITS item formatting. - TJ
-
-  # Make sure that we do not stringify reference arguments by mistake
-  # when looking from slashes
-
-  if ( !ref($value) && $keyword !~ m/(COMMENT)|(HISTORY)/ and 
-      $value =~ s:\s*(?<!\\)/\s*(.*)::         # Identify any '/' not preceded by '\'
-      ) { 
-    my $comment = $1;
-
-    # Recurse to store the comment.  This is a direct (non-method) call to 
-    # keep this method monolithic.  --CED 27-Jun-2003
-    STORE($self,$keyword."_COMMENT",$comment);
-
-  }
-
-  # unescape (unless we are blessed)
-  if (!ref($value)) {
-    $value =~ s:\\\\:\\:g;
-    $value =~ s:\\\/:\/:g;
-  }
+    my @values;
 
   # skip the shenanigans for the normal case
-  # or if we have an Astro::FITS::Header
-  if (UNIVERSAL::isa($value, "Astro::FITS::Header")) {
-    @values = ($value);
-
-  } elsif (ref $value eq 'HASH') {
-    # Convert a hash to a Astro::FITS::Header
-    # If this is a tied hash already just get the object
-    my $tied = tied %$value;
-    if (defined $tied && UNIVERSAL::isa($tied, "Astro::FITS::Header")) {
-      # Just take the object
-      @values = ($tied);
-    } else {
-      # Convert it to a hash
-      my @items = map { new Astro::FITS::Header::Item( Keyword => $_,
-                                                       Value => $value->{$_}
-                                                     ) } keys (%{$value});
-
-      # Create the Header object.
-      @values = (new Astro::FITS::Header( Cards => \@items ));
-
-    }
-
-  } elsif((ref $value eq 'ARRAY') || (length $value > 70) || $value =~ m/\n/s ) {
+  if( (ref $value) || (length $value > 70) || $value =~ m/\n/s ) {
     my @val;
     # @val gets intermediate breakdowns, @values gets line-by-line breakdowns.
-
+    
     # Change multiline strings into array refs
     if (ref $value eq 'ARRAY') {
       @val = @$value;
@@ -1053,7 +763,7 @@ sub STORE {
     } else {
       @val = $value;
     }
-
+    
     # Cut up really long items into multiline strings
     my($val);
     foreach $val(@val) {
@@ -1065,64 +775,12 @@ sub STORE {
     }
   }   ## End of complicated case
   else {
-
-
-
-    @values = ($value);
+    @values = $value;
   }
-
-  # Upper case the relevant item name
+    
   $keyword = uc($keyword);
-  
-  if($keyword eq 'END') {
-      # Special case for END keyword
-      # (drops value on floor, makes sure there is one END at the end)
-      my @index = $self->index($keyword);
-      if( @index != 1   ||   $index[0] != $#{$self->allitems}) {
-	  my $i;
-	  while(defined($i = shift @index)) {
-	      $self->remove($i);
-	  }
-      }
-      unless( @index ) {
-	  my $endcard = new Astro::FITS::Header::Item(Keyword=>'END',
-						      Type=>'END', 
-						      Value=>1);
-	  $self->insert( scalar ($self->allitems) , $endcard );
-      }
-      return;
-      
-  } 
-  
-  if($keyword eq 'SIMPLE') {
-      # Special case for SIMPLE keyword
-      # (sets value correctly, makes sure there is one SIMPLE at the beginning)
-      my @index = $self->index($keyword);
-      if( @index != 1  ||  $index[0] != 0) {
-	  my $i;
-	  while(defined ($i=shift @index)) {
-	      $self->remove($i);
-	  }
-      }
-      unless( @index ) {
-	  my $simplecard = new Astro::FITS::Header::Item(Keyword=>'SIMPLE',
-							 Value=>$values[0],
-							 Type=>'LOGICAL');
-	  $self->insert(0, $simplecard);
-      }
-      return;
-  }
-  
-
-  # Recognise _COMMENT
-  my $havevalue = 1;
-  if ($keyword =~ /_COMMENT$/) {
-    $keyword =~ s/_COMMENT$//;
-    $havevalue = 0;
-  }
-
   my @items = $self->itembyname($keyword);
-
+  
   ## Remove extra items if necessary
   if(scalar(@items) > scalar(@values)) {
     my(@indices) = $self->index($keyword);
@@ -1131,26 +789,26 @@ sub STORE {
       $self->remove( $indices[-$i] );
     }
   }
-
-  ## Allocate new items if necessary
+  
+  ## Allocate items if necessary
   while(scalar(@items) < scalar(@values)) {
 
     my $item = new Astro::FITS::Header::Item(Keyword=>$keyword,Value=>undef);
-    # (No need to set type here; Item does it for us)
+    $item->type('COMMENT') if($Astro::FITS::Header::COMMENT_FIELD{$keyword}
+			      || ((defined $items[0]) && 
+				  (defined $items[0]->type) &&
+				  ($items[0]->type eq 'COMMENT')));
 
     $self->insert(-1,$item);
     push(@items,$item);
   }
-
+  
   ## Set values or comments
   my($i); 
   for $i(0..$#values) {
     if($Astro::FITS::Header::COMMENT_FIELD{$keyword}) {
       $items[$i]->type('COMMENT');
       $items[$i]->comment($values[$i]);
-    } elsif (! $havevalue) {
-      # This is actually just changing the comment
-      $items[$i]->comment($values[$i]);
     } else {
       $items[$i]->type( (($#values > 0) || ref $value) ? 'STRING' : undef);
 
@@ -1158,13 +816,13 @@ sub STORE {
       $items[$i]->type("STRING") if($#values > 0);
     }
   }
-}
+}  
 
 
 # reports whether a key is present in the hash
 sub EXISTS {
   my ($self, $keyword) = @_;
-  return undef unless exists ${$self->{LOOKUP}}{$keyword};
+  return undef unless exists ${$self->{LOOKUP}}{$keyword};  
 }
 
 # deletes a key and value pair
@@ -1187,7 +845,7 @@ sub FIRSTKEY {
   my $self = shift;
   $self->{LASTKEY} = 0;
   $self->{SEENKEY} = {};
-  return undef unless @{$self->{HEADER}};
+  return undef unless defined @{$self->{HEADER}};
   return ${$self->{HEADER}}[0]->keyword();
 }
 
@@ -1227,11 +885,10 @@ it under the same terms as Perl itself.
 
 Alasdair Allan E<lt>aa@astro.ex.ac.ukE<gt>,
 Tim Jenness E<lt>t.jenness@jach.hawaii.eduE<gt>,
-Craig DeForest E<lt>deforest@boulder.swri.eduE<gt>,
-Jim Lewis E<lt>jrl@ast.cam.ac.ukE<gt>
+Craig DeForest E<lt>deforest@boulder.swri.eduE<gt>
 
 =cut
 
 # L A S T  O R D E R S ------------------------------------------------------
 
-1;
+1;                                                                  
