@@ -9,7 +9,6 @@ Astro::FITS::Header::NDF - Manipulate FITS headers from NDF files
   use Astro::FITS::Header::NDF;
 
   $hdr = new Astro::FITS::Header::NDF( Cards => \@cards );
-  $hdr = new Astro::FITS::Header::NDF( Items => \@items );
   $hdr = new Astro::FITS::Header::NDF( ndfID => $indf );
   $hdr = new Astro::FITS::Header::NDF( File => $file );
 
@@ -18,32 +17,22 @@ Astro::FITS::Header::NDF - Manipulate FITS headers from NDF files
 
 =head1 DESCRIPTION
 
-This module makes use of the Starlink L<NDF|NDF> module to read and
-write to an NDF FITS extension or to a C<.HEADER> block in an HDS
-container file.  If the file is found to be an HDS container
-containing multiple NDFs at the top level, either the .HEADER NDF or
-the first NDF containing a FITS header is deemed to be the primary
-header, and all other headers a subsidiary headers indexed by the name
-of the NDF in the container.
+This module makes use of the Starlink L<NDF|NDF> module to read and write to
+and NDF FITS extension.
 
-It stores information about a FITS header block in an object. Takes an
-hash as an argument, with either an array reference pointing to an
-array of FITS header cards, array of C<Astro::FITS::Header::Item>
-objects, or a filename, or (alternatively) an NDF identifier.
-
-Currently, subheader support is readonly.
+It stores information about a FITS header block in an object. Takes an hash as an arguement, with either an array reference pointing to an array of FITS header cards, or a filename, or (alternatively) and NDF identifier.
 
 =cut
 
 use strict;
 use Carp;
-use NDF qw/ :ndf :dat :err :hds /;
+use NDF qw/ :ndf :dat :err /;
 
 use base qw/ Astro::FITS::Header /;
 
 use vars qw/ $VERSION /;
 
-$VERSION = '0.03';
+$VERSION = '0.01';
 
 =head1 METHODS
 
@@ -57,15 +46,8 @@ Reads a FITS header from an NDF.
   $hdr->configure( ndfID => $indf );
   $hdr->configure( File => $filename );
 
-Accepts an NDF identifier or a filename. If both C<ndfID> and C<File> keys
-exist, C<ndfID> key takes priority.
-
-If the file is actually an HDS container, an attempt will be made
-to read a ".HEADER" NDF inside that container (this is the standard
-layout of UKIRT (and some JCMT) data files). If an extension is specified
-explicitly (that is not ".sdf") that path is treated as an explicit path
-to an NDF. If an explicit path is specified no attempt is made to locate
-other NDFs in the HDS container.
+Accepts an NDF identifier or a filename. If both ndfID and File keys
+exist, ndfID key takes priority.
 
 =cut
 
@@ -77,8 +59,7 @@ sub configure {
   my ($indf, $started);
   my $task = ref($self);
 
-  return $self->SUPER::configure(%args) 
-    if exists $args{Cards} or exists $args{Items};
+  return $self->SUPER::configure(%args) if exists $args{Cards};
 
   # Store the definition of good locally
   my $status = &NDF::SAI__OK;
@@ -101,43 +82,8 @@ sub configure {
     my $file = $args{File};
     $file =~ s/\.sdf$//;
 
-    # First we need to find whether we have an HDS container or a
-    # straight NDF. Rather than simply trying an ndf_find on both
-    # (which causes leaks in the NDF system circa 2001) we explicitly
-    # open it using HDS unless it has a "." in it.
-    if ($file =~ /\./) {
-      # an NDF
-      ndf_find(&NDF::DAT__ROOT(), $file, $indf, $status);
-    } else {
-      # Try HDS
-      hds_open( $file, 'READ', my $hdsloc, $status);
-
-      # Find its type
-      dat_type( $hdsloc, my $type, $status);
-
-      if ($status == $good) {
-
-	# If we have an NDF we can simply reopen it
-	# Additionally if we have no description of the component
-	# at all we assume NDF. This overcomes a bug in the acquisition
-	# for SCUBA where a blank type field is used.
-	my $ndffile;
-	if ($type =~ /NDF/i || $type !~ /\w/) {
-	  $ndffile = $file;
-	} else {
-	  # For now simply assume we can find a .HEADER
-	  # in future we could tweak this to default to first NDF
-	  # it finds if no .HEADER
-	  $ndffile = $file . ".HEADER";
-	}
-
-	# Close the HDS file
-	dat_annul( $hdsloc, $status);
-
-	# Open the NDF
-	ndf_find(&NDF::DAT__ROOT(), $ndffile, $indf, $status);
-      }
-    }
+    # Open it
+    ndf_find(&NDF::DAT__ROOT(), $file, $indf, $status);
 
   } else {
 
@@ -223,15 +169,15 @@ sub configure {
 
 =item B<writehdr>
 
-Write a FITS header to an NDF.
+Write a fits header to an NDF.
 
   $hdr->writehdr( ndfID => $indf );
   $hdr->writehdr( File => $file );
 
-Accepts an NDF identifier or a filename.  If both C<ndfID> and C<File> keys
-exist, C<ndfID> key takes priority.
+Accepts an NDF identifier or a filename.  If both ndfID and File keys
+exist, ndfID key takes priority.
 
-Returns C<undef> on error, true if the header was written successfully.
+Returns undef on error, true if the header was written successfully.
 
 =cut
 
@@ -262,25 +208,12 @@ sub writehdr {
     ndf_open(&NDF::DAT__ROOT(), $file, 'UPDATE', 'UNKNOWN',
 	     $ndfid, my $place, $status);
 
-    # If status is bad, try assuming it is a HDS container
-    # with UKIRT style .HEADER component
-    if ($status != $good or $ndfid == 0) {
-      # dont want to contaminate existing status
-      my $lstat = $good;
-      my $hdsfile .= $file . ".HEADER";
-      ndf_open(&NDF::DAT__ROOT(), $hdsfile, 'UPDATE', 'UNKNOWN',
-	       $ndfid, $place, $lstat);
-
-      # flush bad status if we succedded
-      err_annul($status) if $lstat == $good;
-
-    }
 
     # KLUGE : need to get NDF__NOID from the NDF module at some point
     if ($ndfid == 0 && $status == $good) {
       # could create it :-)
       $status = &NDF::SAI__ERROR;
-      err_rep(' ',"File '$file' does not exist to receive the header", $status);
+      err_rep(' ',"File $file does not exist to receive the header", $status);
     }
 
   } else {
@@ -341,7 +274,7 @@ Alasdair Allan E<lt>aa@astro.ex.ac.ukE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2001-2002 Particle Physics and Astronomy Research Council.
+Copyright (C) 2001 Particle Physics and Astronomy Research Council.
 All Rights Reserved.
 
 =cut
